#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// **********************************************************************
// PUCRS/FACIN
// Programa leitura de arquivo PGM
//
// Marcelo Cohen
//
// marcelo.cohen@pucrs.br
// **********************************************************************

typedef int BOOL;

#define FALSE 0
#define TRUE 1


// **********************************************************************
unsigned char *CompactaVetor(unsigned char *VetorDeEntrada, int QtdDeDados, int*QtdSaida)
{
    unsigned char *VetorTemp;
    // inicialmente, pode alocar um vetor igual ao da entrada
    VetorTemp = (unsigned char*) malloc (QtdDeDados);

    // No final redimensiona o vetor.
    unsigned char *VetorFinal;

    // 
    // EXEMPLO: gera uma imagem de saida com as mesmas dimensões da entrada,
    // mas preenchida com sequencias de 0 a 255, até acabar.
    // A cada 5 sequências, aumenta o índice
    printf("Quantidade de dados na entrada: %d\n", QtdDeDados);
    int totalSeqs = QtdDeDados / 255; // quantas seqs de 255 devemos gerar
    int seqFinal  = QtdDeDados % 255; // restante para completar o tam. da entrada

    *QtdSaida = totalSeqs * 2; // 2 bytes cada seq.
    printf("Saida: %d\n", *QtdSaida);
    int index = 0;
    int pos;
    for(pos=0; pos<*QtdSaida; pos+=2) {
        VetorTemp[pos] = index;
        VetorTemp[pos+1] = 255;
        if(pos%10==0) index++;
        if(index > 255) index = 0;
    }
    if(seqFinal != 0) {
        VetorTemp[pos] = index;
        VetorTemp[pos+1] = seqFinal; // o que faltou para completar a entrada
        *QtdSaida+=2; // nesse caso, acrescenta esses dois bytes na qtd. da saida
    }
    VetorFinal = VetorTemp;

    // e retorna
    return VetorFinal;
}
// **********************************************************************
FILE *AbreArquivoPGM(const char *nome)
{
    FILE *arq;
    arq = fopen (nome,"rt");
    if (arq == NULL)
    {
        printf("Problemas na abertura do arquivo %s...\n", nome);
        return NULL;
    }
    printf("Arquivo %s aberto!\n", nome);
    return arq;
}

// **********************************************************************
void LeTamanhoPGM(FILE *arq, int *largura, int *altura)
{
    char aux[30];
    fscanf(arq, "%s", aux);
    if(strcmp(aux, "P5")) {
        printf("Formato inválido: %s\n", aux);
        exit(1);
    }
    //printf("lendo largura e altura...");
    int maxcol; // deve ser 255 em um PGM
    fscanf(arq,"%d", largura);
    fscanf(arq,"%d", altura);
    fscanf(arq,"%d\n", &maxcol);
    printf("Largura: %d  -  Altura %d\n",*largura, *altura );

}

// **********************************************************************
void LeDadosPGM(FILE *arq, unsigned char *V, int qtd)
{
    fread(V, qtd, 1, arq);

    // DEBUG: exibe inicio do vetor na tela
    for(int i=0; i<200; i++) {
        if(i%16 == 0)
            printf("\n%d ", i);
        printf("%02X ",V[i]);
    }
    printf("\n");
}

// Teste: gravando
void GravaSaida(char* nomeSaida, unsigned char *V, int larg, int alt, int qtd)
{
    FILE* arq;
    arq = fopen(nomeSaida, "wb");
    if(arq == NULL) {
        printf("Erro abrindo arquivo de saida: %s\n", nomeSaida);
        exit(1);
    }
    fprintf(arq, "P5\n");
    fprintf(arq, "%d %d\n255\n", larg, alt);
    int pos = 0;
    int total = 0;
    while(pos < qtd) {
       int index = V[pos];
       int cont  = V[pos+1];
       printf("%d - %d\n", index, cont);
       for(int p=0; p<cont; p++)
          fputc(index, arq);
       total += cont;
       pos+=2;
    }
    //fwrite(V, qtd, 1, arq);
    printf("Gravei %d bytes\n", total);
    fclose(arq);
}

// **********************************************************************
int main()
{
    FILE *Arquivo;
    char nomeArquivo[] = "logofacin.pgm";
    int largura, altura;
    unsigned char * VetorDeEntrada;

    Arquivo = AbreArquivoPGM(nomeArquivo);
    if (!Arquivo)
        return 1;
    LeTamanhoPGM(Arquivo, &largura, &altura);

    // Aloca um vetor com o tamanho necessario
    int QtdDeDados = largura * altura;
    VetorDeEntrada = (unsigned char*) malloc (QtdDeDados);
    if (VetorDeEntrada == NULL)
    {
        printf("Faltou memoria!");
        return 2;
    }
    LeDadosPGM(Arquivo, VetorDeEntrada, QtdDeDados);
    // Neste ponto do codigo, o VetorDeEntrada tem os dados lidos do arquivo
    unsigned char *VetorCompactado;
    int QtdSaida;
    VetorCompactado = CompactaVetor(VetorDeEntrada, QtdDeDados, &QtdSaida);

    printf("Total de elementos na saida: %d\n", QtdSaida);
    GravaSaida("saida.pgm", VetorCompactado, largura, altura, QtdSaida);
    printf("FIM.");
    free(VetorDeEntrada);
    free(VetorCompactado);
    return 0;
}
